
#include <DistanceConstraint.h>


extern "C" void ExecuteDistanceConstraint(dim3 block, dim3 grid, float4* pos, float4* springs, unsigned int springCount, int iterations, float dt);
extern "C" void ExecuteBatchedDistanceConstraint(dim3 block, dim3 grid, float4 *pos, float4 **springArrays, int *springArraySizes, unsigned int springArrayCount, int iterations, float dt);

// static
void DistanceConstraint::Execute(void *pos, void *springs, unsigned int springCount, int iterations, float dt)
{
    dim3 block(clampValue(springCount, 0, 256), 1, 1);
    //dim3 block(springCount < 256 ? springCount: 256, 1, 1);
    int gridX = springCount / block.x;
    if ((springCount % block.x) > 0)
        gridX++;
    dim3 grid(gridX, 1, 1);

    ExecuteDistanceConstraint(block, grid, (float4 *)pos, (float4 *)springs, springCount, iterations, dt);
}

// Lanzamos el kernel una sola vez, que iterar por todos los sets de springs
// con esto nos ahorramos el overhead de lanzar un kernel por cada set de springs
// el tradeoff es que algunos threads no haran nada si springCount < largestArraySize (Max Spring Size)
void DistanceConstraint::ExecuteBatched(void *pos, void **springArrays, void *springArraySizes, unsigned int springArrayCount, int largestArraySize, int iterations, float dt)
{
    dim3 block(clampValue(largestArraySize, 0, 256), 1, 1);
    //dim3 block(springCount < 256 ? springCount: 256, 1, 1);
    int gridX = largestArraySize / block.x;
    if ((largestArraySize % block.x) > 0)
        gridX++;
    dim3 grid(gridX, 1, 1);

    ExecuteBatchedDistanceConstraint(block, grid, (float4 *)pos, (float4 **)springArrays, (int *)springArraySizes, springArrayCount, iterations, dt);
}
